@@ -270,8 +270,9 @@ class Blocks(EventDispatcher):
 				widget.widget_id = id
 		
 		widget.build_desc = desc
-		self.build_attributes(widget,desc)
-		self.build_rest(widget,desc)
+		self.build_attributes(widget, desc)
+		self.build_rest(widget, desc)
+		self.buildBinds(widget, desc)
 		return widget
 		
 	def build_attributes(self, widget:Widget,desc,t=None):
@@ -297,19 +298,30 @@ class Blocks(EventDispatcher):
 		pos = 0
 		for pos,sw in enumerate(desc.get('subwidgets',[])):
 			b = Blocks()
+			if isinstance(sw, str):
+				# w = Blocks.getWidgetById(sw, from_widget=widget)
+				w = Blocks.findWidget(sw, from_widget=widget)
+				if w:
+					widget.add_widget(w)
+				continue
+				
 			kw = self.valueExpr(sw.copy(), 
 						localnamespace={'self':widget})
 			w = b.widgetBuild(kw)
-			widget.add_widget(w)
+			if w:
+				widget.add_widget(w)
 
+
+	def buildBinds(self, widget:Widget, desc:dict):
 		for b in desc.get('binds',[]):
 			kw = self.valueExpr(b.copy(), \
 						localnamespace={'self':widget})
 			self.buildBind(widget,kw)
 
-	def buildBind(self, widget:Widget, desc):
+	def buildBind(self, widget:Widget, desc:dict):
 		wid = desc.get('wid','self')
-		w = Blocks.getWidgetById(desc.get('wid','self'),from_widget=widget)
+		# w = Blocks.getWidgetById(desc.get('wid','self'),from_widget=widget)
+		w = Blocks.findWidget(desc.get('wid','self'),from_widget=widget)
 		if not w:
 			Logger.info('Block: %s %s',desc.get('wid','self'),
 							'not found via Blocks.getWidgetById()')
@@ -385,7 +397,8 @@ class Blocks(EventDispatcher):
 	def get_target(self, widget:Widget, desc):
 		if not desc.get('target'):
 			return None
-		return Blocks.getWidgetById(desc.get('target'),from_widget=widget)
+		# return Blocks.getWidgetById(desc.get('target'),from_widget=widget)
+		return Blocks.findWidget(desc.get('target'),from_widget=widget)
 
 	def blocksAction(self, widget:Widget, desc, *args):
 		target = self.get_target(widget, desc)
@@ -533,7 +546,8 @@ class Blocks(EventDispatcher):
 			w = args[0]
 		target = desc.get('target')
 		if target:
-			w = self.getWidgetById(target, from_widget=widget)
+			# w = Blocks.getWidgetById(target, from_widget=widget)
+			w = Blocks.findWidget(target, from_widget=widget)
 		if w is None:
 			print('get_rtdata():w is None', desc)
 			return {}
@@ -664,69 +678,93 @@ class Blocks(EventDispatcher):
 		return doit(desc)
 	
 	@classmethod
-	def getWidgetById(self, id:str, from_widget:Widget=None) -> Widget:
+	def findWidget(self, id:str, from_widget:Widget=None) -> Widget:
+		"""
+		-:find up direct, find widget in ancestor
+		+ or empty: find down direct, find widget in descendant
+		@:find widget by class
+		$ or empty:find widget by id
+		.:more than one step.
+
+		"""
 		def find_widget_by_id(id, from_widget):
 			if id=='self':
 				return from_widget
 			if hasattr(from_widget,'widget_id'):
 				if from_widget.widget_id == id:
 					return from_widget
-				if id[0] == '-' and from_widget.widget_id == id[1:]:
-					return from_widget
 			if hasattr(from_widget, id):
 				w = getattr(from_widget,id)
 				return w
+			return None
 
-			if id[0] == '-':
-				print('find_widget_by_id(), id=', id, 
-							'widget=', from_widget)
-				if isinstance(from_widget, WindowBase):
-					return None
-				if from_widget.parent:
-					return find_widget_by_id(id, 
-							from_widget=from_widget.parent)
-				return None
-			else:
+		def find_widget_by_class(klass, from_widget):
+			if from_widget.__class__.__name__ == klass:
+				return from_widget
+			for k, v in from_widget.__dict__.items():
+				if isinstance(v, Widget):
+					if v.__class__.__name__ == klass:
+						return v
+			return None
+		
+		def _find_widget(name, from_widget, dir='down', 
+					find_func=find_widget_by_id):
+			w = find_func(name, from_widget)
+			if w:
+				return w
+			if dir == 'down':
 				children = [i for i in from_widget.children]
 				if hasattr(from_widget, 'get_subwidgets'):
 					children = from_widget.get_subwidgets()
 				for c in children:
-					ret = find_widget_by_id(id,from_widget=c)
+					ret = find_func(id,from_widget=c)
 					if ret:
 						return ret
+			else:
+				if isinstance(from_widget, WindowBase):
+					return None
+				if from_widget.parent:
+					return find_func(id, from_widget=from_widget.parent)
 			return None
 
-		ids = id.split('.')
-		app = App.get_running_app()
-		fid = ids[0]
-		if fid == '/self' or fid == 'root':
-			from_widget = app.root
-			if len(ids) == 1:
-				return from_widget
-			ids = ids[1:]
-		if fid == 'Window':
-			from_widget == Window
-			if len(ids) == 1:
-				return from_widget
-			ids = ids[1:]
-		if fid == 'app':
-			return app
+		def find_widget(step, from_widget):
+			dir = 'down'
+			if step[0:1] == '-':
+				dir = 'up'
+				step = step[1:]
+			find_func = find_widget_by_id
+			if step[0:1] == '@':
+				find_func = find_widget_by_class
+			return _find_widget(step, from_widget=from_widget, 
+						dir=dir, find_func=find_func)
 
-		if from_widget is None:
-			from_widget = app.root
-		for id in ids:
-			w = find_widget_by_id(id,from_widget=from_widget)
-			if w is None \
-					and id == ids[0] \
-					and Window.fullscreen == True \
-					and app.root != app.fs_widget:
-				w = find_widget_by_id(id, app.fs_widget)
+		steps = id.split('.')
+		w = from_widget
+		for i, s in enumerate(steps):
+			if i==0:
+				app = App.get_running_app()
+				fid = s
+				if fid == '/self' or fid == 'root':
+					w = app.root
+					if len(steps) == 1:
+						return from_widget
+					continue
+				if fid == 'Window':
+					w = Window
+					if len(steps) == 1:
+						return w
+					continue
+				if fid == 'app':
+					return app
 
-			if w is None:
-				print(id, 'not found ....')
+			w = find_widget(s, w)
+			if not w:
 				return None
-			from_widget = w
 		return w
+			
+	@classmethod
+	def getWidgetById(self, id:str, from_widget:Widget=None) -> Widget:
+		return Blocks.findWidget(id, from_widget=from_widget)
 
 	def on_built(self,v=None):
 		return
